import urllib.parse
from dataclasses import dataclass
from datetime import datetime, timezone

import lol_id_tools as lit
from lol_dto.classes.game import (
    LolGame,
    LolGamePlayer,
    LolGameTeamEndOfGameStats,
)
from lol_dto.classes.sources.riot_lol_api import RiotGameSource

from leaguepedia_parser.transmuters.game_players import LeaguepediaPlayerIdentifier


@dataclass
class LeaguepediaGameIdentifier:
    gameId: str
    matchId: str

    matchHistoryUrl: str
    overviewPage: str

    tournamentName: str


@dataclass
class LeaguepediaTeamIdentifier:
    name: str = None


def transmute_game(source_dict: dict) -> LolGame:
    """
    Transforms a ScoreboardGames row into a LolGame

    Some fields like team gold and kills are not present. Get_game_details should be used for it.
    """

    game = LolGame(
        start=datetime.fromisoformat(source_dict["DateTime UTC"])
        .replace(tzinfo=timezone.utc)
        .isoformat(timespec="seconds"),
        gameInSeries=int(source_dict["N GameInMatch"]),
        patch=source_dict["Patch"],
        duration=int(float(source_dict["Gamelength Number"] or 0) * 60),
        vod=source_dict["VOD"],
        winner="BLUE" if source_dict["Winner"] == "1" else "RED",
    )

    setattr(
        game.sources,
        "leaguepedia",
        LeaguepediaGameIdentifier(
            gameId=source_dict["GameId"],
            matchId=source_dict["MatchId"],
            matchHistoryUrl=source_dict["MatchHistory"],
            overviewPage=source_dict["OverviewPage"],
            tournamentName=source_dict["Tournament"],
        ),
    )

    for team, idx in [(game.teams.BLUE, 1), (game.teams.RED, 2)]:
        team.bans = [
            lit.get_id(champion_name, object_type="champion")
            for champion_name in source_dict[f"Team{idx}Bans"].split(",")
        ]

        team.endOfGameStats = LolGameTeamEndOfGameStats(
            turretKills=int(source_dict[f"Team{idx}Towers"] or 0),
            dragonKills=int(source_dict[f"Team{idx}Dragons"] or 0),
            riftHeraldKills=int(source_dict[f"Team{idx}RiftHeralds"] or 0),
            baronKills=int(source_dict[f"Team{idx}Barons"] or 0),
        )

        for player_idx, champion_name in enumerate(
            source_dict[f"Team{idx}Picks"].split(",")
        ):
            player_object = LolGamePlayer(
                championId=lit.get_id(champion_name, object_type="champion"),
            )

            setattr(
                player_object.sources,
                "leaguepedia",
                LeaguepediaPlayerIdentifier(
                    gameName=source_dict[f"Team{idx}Players"].split(",")[idx]
                ),
            )

            team.players.append(player_object)

        setattr(
            team.sources,
            "leaguepedia",
            LeaguepediaTeamIdentifier(name=source_dict[f"Team{idx}"]),
        )

    # For Riot API games, I directly parse the URL for the game to have its actual identifiers.
    if "gameHash" in source_dict["MatchHistory"]:
        parsed_url = urllib.parse.urlparse(
            urllib.parse.urlparse(source_dict["MatchHistory"]).fragment
        )

        query = urllib.parse.parse_qs(parsed_url.query)
        platform_id, game_id = parsed_url.path.split("/")[1:]
        game_hash = query["gameHash"][0]

        setattr(
            game.sources,
            "riotLolApi",
            RiotGameSource(gameId=game_id, platformId=platform_id, gameHash=game_hash),
        )

    return game



from typing import List
from lol_dto.classes.game.lol_game import LolPickBan
import lol_id_tools as lit

from leaguepedia_parser.transmuters.field_names import picks_bans_fields


def transmute_picks_bans(input_dict) -> List[LolPickBan]:
    pb_list = []

    for field in picks_bans_fields:
        pb_list.append(
            LolPickBan(
                championId=lit.get_id(input_dict[field], object_type="champion"),
                isBan="Ban" in field,
                team="BLUE" if "Team1" in field else "RED",
            )
        )

    return pb_list



import dataclasses
from typing import Optional
from leaguepedia_parser.site.leaguepedia import leaguepedia


@dataclasses.dataclass
class TeamAssets:
    thumbnail_url: str
    logo_url: str
    long_name: str  # Aka display name


def get_all_team_assets(team_link: str) -> TeamAssets:
    """

    Args:
        team_link: a field coming from Team1/Team2 in ScoreboardGames

    Returns:
        A TeamAssets object

    """
    result = leaguepedia.site.client.api(
        action="query",
        format="json",
        prop="imageinfo",
        titles=f"File:{team_link}logo square.png|File:{team_link}logo std.png",
        iiprop="url",
    )

    pages = result["query"]["pages"]

    urls = []
    for v in pages.values():
        urls.append(v["imageinfo"][0]["url"])

    long_name = leaguepedia.site.cache.get("Team", team_link, "link")

    return TeamAssets(
        thumbnail_url=urls[1],
        logo_url=urls[0],
        long_name=long_name,
    )


def get_team_logo(team_name: str, _retry=True) -> str:
    """
    Returns the team logo URL

    Params:
        team_name: Team name, usually gotten from the game dictionary
        _retry: whether or not to get the team’s full name from Leaguepedia if it was not understood

    Returns:
        URL pointing to the team’s logo
    """
    return _get_team_asset(f"File:{team_name}logo square.png", team_name, _retry)


def get_team_thumbnail(team_name: str, _retry=True) -> str:
    """
    Returns the team thumbnail URL

    Params:
        team_name: Team name, usually gotten from the game dictionary
        _retry: whether or not to get the team’s full name from Leaguepedia if it was not understood

    Returns:
        URL pointing to the team’s thumbnail
    """
    return _get_team_asset(f"File:{team_name}logo std.png", team_name, _retry)


def _get_team_asset(asset_name: str, team_name: str, _retry=True) -> str:
    """
    Returns the team thumbnail URL

    Params:
        team_name: Team name, usually gotten from the game dictionary
        _retry: whether or not to get the team’s full name from Leaguepedia if it was not understood

    Returns:
        URL pointing to the team’s logo
    """
    result = leaguepedia.site.client.api(
        action="query",
        format="json",
        prop="imageinfo",
        titles=asset_name,
        iiprop="url",
    )

    try:
        url = None
        pages = result.get("query").get("pages")
        for k, v in pages.items():
            url = v.get("imageinfo")[0].get("url")

    except (TypeError, AttributeError):
        # This happens when the team name was not properly understood.
        if _retry:
            return get_team_logo(get_long_team_name_from_trigram(team_name), False)
        else:
            raise Exception("Logo not found for the given team name")

    return url


def get_long_team_name_from_trigram(
    team_abbreviation: str,
    event_overview_page: str = None,
) -> Optional[str]:
    """
    Returns the long team name for the given team abbreviation using Leaguepedia’s search pages

    Only issues a query the first time it is called, then stores the data in a cache
    There is no cache timeout at the moment

    Args:
        team_abbreviation: A team name abbreviation, like IG or RNG
        event_overview_page: The overviewPage field of the tournament, useful for disambiguation

    Returns:
        The long team name, like "Invictus Gaming" or "Royal Never Give Up"
    """

    # We use only lowercase team abbreviations for simplicity
    team_abbreviation = team_abbreviation.lower()

    if event_overview_page:
        return leaguepedia.site.cache.get_team_from_event_tricode(
            event_overview_page, team_abbreviation
        )

    else:
        return leaguepedia.site.cache.get("Team", team_abbreviation, "link")



from dataclasses import dataclass
from typing import List
from lol_dto.classes.game import LolGame, LolGamePlayer
import lol_id_tools as lit


role_translation = {"1": "TOP", "2": "JGL", "3": "MID", "4": "BOT", "5": "SUP"}


@dataclass
class LeaguepediaPlayerIdentifier:
    gameName: str = None  # Defined here because it’s a leaguepedia-specific field
    name: str = None  # Current player name
    irlName: str = None  # IRL name of the player
    country: str = None  # Country of origin
    birthday: str = None  # YYYY-MM-DD
    pageId: int = None  # Page ID on leaguepedia


def add_players(
    game: LolGame, players: List[dict], add_page_id: bool = False
) -> LolGame:
    """
    Adds additional player information from ScoreboardPlayers
    """

    for idx, team in enumerate(game.teams):
        team_side_leaguepedia = "1" if idx == 0 else "2"

        for player_idx, game_player in enumerate(team.players):
            game_player: LolGamePlayer

            try:
                # We get the player object from the Leaguepedia players list
                player_latest_data = next(
                    p
                    for p in players
                    if p["Side"] == team_side_leaguepedia
                    and lit.get_id(p["Champion"], object_type="champion")
                    == game_player.championId
                )
            except StopIteration:
                # Since we cannot get the role properly, we try to infer it from the index
                game_player.role = list(role_translation.values())[player_idx]
                continue

            game_player.role = role_translation[player_latest_data["gameRoleNumber"]]

            leaguepedia_identifier = LeaguepediaPlayerIdentifier(
                name=player_latest_data.get("currentGameName"),
                irlName=player_latest_data.get("irlName"),
                country=player_latest_data.get("Country"),
                birthday=player_latest_data.get("Birthdate"),
            )

            if add_page_id:
                leaguepedia_identifier.pageId = int(player_latest_data["pageId"])

            setattr(game_player.sources, "leaguepedia", leaguepedia_identifier)

    return game



# Those are the field names of the cargo tables of leaguepedia

# Tournament
tournaments_fields = {
    "Name",
    "DateStart",
    "Date",
    "Region",
    "League",
    "Rulebook",
    "TournamentLevel",
    "IsQualifier",
    "IsPlayoffs",
    "IsOfficial",
    "OverviewPage",
}

# Game
game_fields = {
    "GameId",
    "MatchId",
    "Tournament",
    "Team1",
    "Team2",
    "Winner",
    "Gamelength_Number",
    "DateTime_UTC",
    "Team1Score",
    "Team2Score",
    "Team1Bans",
    "Team2Bans",
    "Team1Picks",
    "Team2Picks",
    "Team1Players",
    "Team2Players",
    "Team1Dragons",
    "Team2Dragons",
    "Team1Barons",
    "Team2Barons",
    "Team1Towers",
    "Team2Towers",
    "Team1RiftHeralds",
    "Team2RiftHeralds",
    "Team1Inhibitors",
    "Team2Inhibitors",
    "Patch",
    "MatchHistory",
    "VOD",
    "Gamename",
    "N_GameInMatch",
    "OverviewPage",
}


# Game Player
game_players_fields = {
    "ScoreboardPlayers.Name=gameName",
    "ScoreboardPlayers.Role_Number=gameRoleNumber",
    "ScoreboardPlayers.Champion",
    "ScoreboardPlayers.Side",
    "Players.Name=irlName",
    "Players.Country",
    "Players.Birthdate",
    "Players.ID=currentGameName",
    # "Players.Image",
    # "Players.Team=currentTeam",
    # "Players.Role=currentRole",
    # "Players.SoloqueueIds",
}


# Ordered in the draft order for pro play as of June 2020
picks_bans_fields = [
    "Team1Ban1",
    "Team2Ban1",
    "Team1Ban2",
    "Team2Ban2",
    "Team1Ban3",
    "Team2Ban3",
    "Team1Pick1",
    "Team2Pick1",
    "Team2Pick2",
    "Team1Pick2",
    "Team1Pick3",
    "Team2Pick3",
    "Team2Ban4",
    "Team1Ban4",
    "Team2Ban5",
    "Team1Ban5",
    "Team2Pick4",
    "Team1Pick4",
    "Team1Pick5",
    "Team2Pick5",
]



from mwrogue.esports_client import EsportsClient


class LeaguepediaSite:
    """A ghost loaded class that handles Leaguepedia connection and some caching.

    Full documentation: https://lol.fandom.com/Help:API_Documentation
    """

    def __init__(self, limit=500):
        self._site = None
        self.limit = limit

    @property
    def site(self):
        if not self._site:
            self._load_site()

        return self._site

    def _load_site(self):
        """Creates site class fields.

        Used for ghost loading the class during package import.
        """
        # If not, we create the self.client object as our way to interact with the wiki
        self._site = EsportsClient("lol")

    def query(self, **kwargs) -> list:
        """Issues a cargo query to leaguepedia.

        Params are usually:
            tables, join_on, fields, order_by, where

        Returns:
            List of rows from the query.
        """
        result = []

        # We check if we hit the API limit
        while len(result) % self.limit == 0:
            result.extend(
                self.site.cargo_client.query(
                    limit=self.limit, offset=len(result), **kwargs
                )
            )

            # If the cargoquery is empty, we stop the loop
            if not result:
                break

        return result


# Ghost loaded instance shared by all other classes
leaguepedia = LeaguepediaSite()



from leaguepedia_parser.parsers.game_parser import (
    get_regions,
    get_tournaments,
    get_games,
    get_game_details,
)
from leaguepedia_parser.parsers.team_parser import (
    get_team_logo,
    get_long_team_name_from_trigram,
    get_team_thumbnail,
    get_all_team_assets
)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['leaguepedia_parser',
 'leaguepedia_parser.parsers',
 'leaguepedia_parser.site',
 'leaguepedia_parser.transmuters']

package_data = \
{'': ['*']}

install_requires = \
['lol-dto>=2.0.0a4,<3.0.0',
 'lol-id-tools>=1.7.1,<2.0.0',
 'mwclient>=0.10.1,<0.11.0',
 'mwrogue>=0.0.14,<0.0.15']

setup_kwargs = {
    'name': 'leaguepedia-parser',
    'version': '2.0.0a1',
    'description': 'A parser for Leaguepedia data',
    'long_description': None,
    'author': 'Tolki',
    'author_email': 'gary.mialaret@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/mrtolkien/leaguepedia_parser',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
